@@ -4,6 +4,12 @@
 Базовый класс, описывающий бизнес-сценарии использования Полиматики.
 Используемые переменные класса:
 
+# Флаг работы в Jupiter Notebook
+self.jupiter
+
+# Текст ошибки присвается в случае аварийного завершения работы; может быть удобно при работе в Jupiter Notebook
+self.current_exception
+
 # Версия сервера Полиматики; например, '5.6'
 self.polymatica_version
 
@@ -14,18 +20,9 @@ self.full_polymatica_version
 # По-умолчанию "ru"
 self.language
 
-# URL стенда Полиматики
-self.url
-
-# Флаг работы в Jupiter Notebook
-self.jupiter
-
 # Таймаут выполнения запросов
 self.timeout
 
-# Текст ошибки присвается в случае аварийного завершения работы; может быть удобно при работе в Jupiter Notebook
-self.current_exception
-
 # Логин пользователя Полиматики
 self.login
 
@@ -59,6 +56,15 @@ self.total_row
 # Идентификатор активного модуля графиков
 self.graph_module_id
 
+# URL стенда Полиматики для выполнения API-запросов
+self.url
+
+# URL стенда Полиматики для обращения к ресурсам
+self.resources_url
+
+# URL стенда Полиматики для логина
+self.login_url
+
 # Идентификатор сессии
 self.session_id
 
