@@ -5,51 +5,44 @@
 import re
 import requests
 import datetime
-import os
-from typing import Dict, Tuple
+from typing import Dict, Union, Callable
+from .common import MONTHS, WEEK_DAYS, datetime_type
 
 
 def request_asserts(response: Dict, r: requests.models.Response) -> bool:
     """
-    Проверка ответов сервера
-    :param response: (dict) ответ сервера, r.json()
+    Проверка ответов сервера.
+    :param response: (dict) ответ сервера в формате json (по сути, это r.json()).
     :param r: <class 'requests.models.Response'>
-    :return: True / False
+    :return: True, если все проверки прошли успешно, иначе будет сгенерирован AssertionError.
     """
     # парсинг ответа
-    resp_queries = response.get("queries")
-    resp_error = response.get("error")
-    resp_err_code = resp_error.get("code")
-    resp_queries = next(iter(resp_queries))  # [0] element in vector
+    response_queries = response.get("queries")
+    assert len(response_queries) > 0, 'No field "queries" in response!'
+    resp_queries = next(iter(response_queries))  # [0] element in vector
     resp_command = resp_queries.get("command")
     resp_command_err = resp_command.get("error")
 
-    assert len(resp_queries) > 0, resp_error
-    assert resp_command, resp_command_err
+    assert resp_command, str(resp_command_err)
     assert r.status_code == 200, "Response code != 200"
-    assert resp_err_code == 0, "error code in response != 0. Err code: %s" % resp_err_code
+
     if "error" in resp_command:
         resp_command_err_code = resp_command_err.get("code")
         resp_command_err_message = resp_command_err.get("message")
-
-        assert resp_command_err_code == 0, "ERROR in response: %s" % resp_command_err_message
+        assert resp_command_err_code == 0, "Error in response: {}".format(resp_command_err_message)
 
     if ("error" in resp_command) and ("status" in resp_command):
         resp_command_status = resp_command.get("status")
         resp_command_status_code = resp_command_status.get("code")
         resp_command_status_message = resp_command_status.get("message")
-
-        assert resp_command_status_code == 0, \
-            "ERROR in response: %s" % resp_command_status_message
+        assert resp_command_status_code == 0, "Error in response: {}".format(resp_command_status_message)
 
     if ("error" in resp_command) and ("datasources" in resp_command):
-        resp_command_datasources = resp_command.get("datasources")
-        resp_command_datasources = next(iter(resp_command_datasources))  # [0] element in vector
+        resp_command_datasources = next(iter(resp_command.get("datasources")))  # [0] element in vector
         datasources_status = resp_command_datasources.get("status")
         datasources_status_code = datasources_status.get("code")
         resp_command_status_message = datasources_status.get("message")
-
-        assert datasources_status_code == 0, "ERROR in response: %s" % resp_command_status_message
+        assert datasources_status_code == 0, "Error in response: {}".format(resp_command_status_message)
 
     return True
 
@@ -139,7 +132,7 @@ def checks(self, func_name, *args):
             raise ValueError(error_msg)
         return True
     elif func_name == "put_dim_filter":
-        filter_name, start_date, end_date, months, week_days = args[0], args[1], args[2], args[3], args[4]
+        filter_name, start_date, end_date, filter_field_format = args[0], args[1], args[2], args[3]
         if (filter_name is None) and (start_date is None and end_date is None):
             raise ValueError("If you don't filter one value by param filter_name,"
                              " please assign value to args start_date AND end_date!")
@@ -153,76 +146,121 @@ def checks(self, func_name, *args):
         # Заполнение списка dates_list в зависимости от содержания параметров filter_name, start_date, end_date
         # заполнить список для недельного интервала
         if (filter_name is None) and (start_date is not None and end_date is not None):
-            if (start_date in week_days) and (end_date in week_days):
-                start_ind = week_days.index(start_date)
-                end_ind = week_days.index(end_date)
+            # заполнение списка недельного интервала
+            if (start_date in WEEK_DAYS) and (end_date in WEEK_DAYS):
+                start_ind = WEEK_DAYS.index(start_date)
+                end_ind = WEEK_DAYS.index(end_date)
                 if start_ind > end_ind:
                     raise ValueError("Start week day can not be more than the end week day!")
-                dates_list = week_days[start_ind:end_ind + 1]
-            # заполнить список для месячного интервала
-            elif (start_date in months) and (end_date in months):
-                start_ind = months.index(start_date)
-                end_ind = months.index(end_date)
+                dates_list = WEEK_DAYS[start_ind:end_ind + 1]
+            # заполнение списка месячного интервала
+            elif (start_date in MONTHS) and (end_date in MONTHS):
+                start_ind = MONTHS.index(start_date)
+                end_ind = MONTHS.index(end_date)
                 if start_ind > end_ind:
                     raise ValueError("Start month can not be more than the end month!")
-                dates_list = months[start_ind:end_ind + 1]
-            # Заполнение списка с интервалом числовых дат
+                dates_list = MONTHS[start_ind:end_ind + 1]
+            # заполнение списка с интервалом числовых дат
             elif isinstance(start_date, int) and isinstance(end_date, int):
                 if start_date > end_date:
                     raise ValueError("Start date can not be more than the end date!")
                 end_date += 1
-                dates_list = [str(x) for x in range(start_date, end_date)]  # list with dates to filter
-            # заполнение списка в формате ЧЧ.ММ.ГГГГ, ЧЧ:ММ:СС
-            elif (re.search(r'(\d+.\d+.\d+, \d+:\d+:\d+)', start_date) is not None) and \
-                    (re.search(r'(\d+.\d+.\d+, \d+:\d+:\d+)', end_date) is not None):
-                start = datetime.datetime.strptime(start_date, "%d.%m.%Y, %H:%M:%S")
-                end = datetime.datetime.strptime(end_date, "%d.%m.%Y, %H:%M:%S")
-                if start > end:
-                    raise ValueError("Start date can not be more than the end date!")
+                dates_list = [str(x) for x in range(start_date, end_date)]
+            # заполнение списка в одном из форматов (год и день недели можно менять местами):
+            # "ДД.ММ.ГГГГ"
+            # "ДД-ММ-ГГГГ"
+            # "ДД.ММ.ГГГГ ЧЧ:ММ:СС"
+            # "ДД-ММ-ГГГГ ЧЧ:ММ:СС"
+            elif isinstance(start_date, str) and isinstance(end_date, str):
+                def get_start_end_date(start: str, end: str) -> Callable:
+                    """
+                    Возвращает функцию получения начальной и конечной даты (по сути, является обёрткой над функцией).
+                    """
+                    start_date, end_date = start, end
+                    def get_date_range(pattern: str, current_format: str) -> Union[datetime_type, None]:
+                        """
+                        Получение начальной и конечной даты по заданному шаблону (паттерну) и формату.
+                        Генерирует ошибку, если начальная дата больше конечной.
+                        Если заданные даты начала и конца не подходят под шаблон - вернётся None, None.
+                        ВАЖНО: подразумевается, что формат начальной и конечной даты одинаков.
+                        :param pattern: шаблон для регулярного выражения
+                        :param current_format: формат даты в Полиматике
+                        """
+                        reg = re.compile(pattern)
+                        if reg.match(start_date) and reg.match(end_date):
+                            try:
+                                start = datetime.datetime.strptime(start_date, current_format)
+                                end = datetime.datetime.strptime(end_date, current_format)
+                            except Exception:
+                                error_msg = 'Could not convert string to datetime format! ' \
+                                    'start_date: {}, end_date: {}'.format(start_date, end_date)
+                                raise ValueError(error_msg)
+                            if start > end:
+                                raise ValueError('Start date can not be more than the end date!')
+                            return start, end
+                        return None, None
+                    return get_date_range
+
+                # функция генерирования начальной и конечной даты
+                date_range_func = get_start_end_date(start_date, end_date)
+
+                # описание паттернов и форматов
+                # FIXME Внимание, говнокод (говнорегулярки)! Подумать, как можно сделать лучше.
+                patterns = (
+                    (
+                        r'^([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1})\.([0]?[1-9]{1}|1[0-2]{1})\.[1-9]{1}[0-9]{3}$',
+                        '%d.%m.%Y'
+                    ),
+                    (
+                        r'^[1-9]{1}[0-9]{3}\.([0]?[1-9]{1}|1[0-2]{1})\.([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1})$',
+                        '%Y.%m.%d'
+                    ),
+                    (
+                        r'^([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1})\-([0]?[1-9]{1}|1[0-2]{1})\-[1-9]{1}[0-9]{3}$',
+                        '%d-%m-%Y'
+                    ),
+                    (
+                        r'^[1-9]{1}[0-9]{3}\-([0]?[1-9]{1}|1[0-2]{1})\-([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1})$',
+                        '%Y-%m-%d'
+                    ),
+                    (
+                        r'^([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1})\.([0]?[1-9]{1}|1[0-2]{1})\.[1-9]{1}[0-9]{3} ([0]?[0-9]{1}|1[0-9]{1}|2[0-3]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})',
+                        '%d.%m.%Y %H:%M:%S'
+                    ),
+                    (
+                        r'^[1-9]{1}[0-9]{3}\.([0]?[1-9]{1}|1[0-2]{1})\.([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1}) ([0]?[0-9]{1}|1[0-9]{1}|2[0-3]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})',
+                        '%Y.%m.%d %H:%M:%S'
+                    ),
+                    (
+                        r'^([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1})\-([0]?[1-9]{1}|1[0-2]{1})\-[1-9]{1}[0-9]{3} ([0]?[0-9]{1}|1[0-9]{1}|2[0-3]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})',
+                        '%d-%m-%Y %H:%M:%S'
+                    ),
+                    (
+                        r'^[1-9]{1}[0-9]{3}\-([0]?[1-9]{1}|1[0-2]{1})\-([0]?[1-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-1]{1}) ([0]?[0-9]{1}|1[0-9]{1}|2[0-3]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})\:([0]?[0-9]{1}|1[0-9]{1}|2[0-9]{1}|3[0-9]{1}|4[0-9]{1}|5[0-9]{1})',
+                        '%Y-%m-%d %H:%M:%S'
+                    )
+                )
+
+                # определяем подходящий формат
+                start, end = None, None
+                for pattern in patterns:
+                    current_start, current_end = date_range_func(pattern[0], pattern[1])
+                    if current_start and current_end:
+                        start, end = current_start, current_end
+                        break
+                else:
+                    raise ValueError("Unknown date format! start_date: {}, end_date: {}".format(start_date, end_date))
+
+                # получаем формат хранения дат в Полиматике, если он не задан пользователем
+                date_format_pattern = filter_field_format or self.get_current_datetime_format()
+
+                # заполняем dates_list
                 step = datetime.timedelta(days=1)
                 while start <= end:
-                    date = start.strftime("%d.%m.%Y, %H:%M:%S")
-                    dates_list.append(date)
-                    start += step
-            # заполнение списка в формате ГГГГ-ММ-ДД ЧЧ:ММ:СС
-            elif (re.search(r'(\d+-\d+-\d+ \d+:\d+:\d+)', start_date) is not None) and \
-                    (re.search(r'(\d+-\d+-\d+ \d+:\d+:\d+)', end_date) is not None):
-                start = datetime.datetime.strptime(start_date, "%Y-%m-%d %H:%M:%S")
-                end = datetime.datetime.strptime(end_date, "%Y-%m-%d %H:%M:%S")
-                if start > end:
-                    raise ValueError("Start date can not be more than the end date!")
-                step = datetime.timedelta(days=1)
-                while start <= end:
-                    date = start.strftime("%Y-%m-%d %H:%M:%S")
-                    dates_list.append(date)
-                    start += step
-            # заполнение списка в формате ГГГГ-ММ-ЧЧ
-            elif (re.search(r'(\d+-\d+-\d+)', start_date) is not None) and \
-                    (re.search(r'(\d+-\d+-\d+)', end_date) is not None):
-                start = datetime.datetime.strptime(start_date, "%Y-%m-%d")
-                end = datetime.datetime.strptime(end_date, "%Y-%m-%d")
-                if start > end:
-                    raise ValueError("Start date can not be more than the end date!")
-                step = datetime.timedelta(days=1)
-                while start <= end:
-                    date = start.strftime("%Y-%m-%d")
-                    dates_list.append(date)
-                    start += step
-            # заполнение списка в формате ЧЧ.ММ.ГГГГ
-            elif (re.search(r'(\d+.\d+.\d+, \d+:\d+:\d+)', start_date) is None) and \
-                    (re.search(r'(\d+.\d+.\d+)', start_date) is not None) and \
-                    (re.search(r'(\d+.\d+.\d+)', end_date) is not None):
-                start = datetime.datetime.strptime(start_date, "%d.%m.%Y")
-                end = datetime.datetime.strptime(end_date, "%d.%m.%Y")
-                if start > end:
-                    raise ValueError("Start date can not be more than the end date!")
-                step = datetime.timedelta(days=1)
-                while start <= end:
-                    date = start.strftime("%d.%m.%Y")
-                    dates_list.append(date)
+                    dates_list.append(start.strftime(date_format_pattern))
                     start += step
             else:
-                raise ValueError("Unknown date format!")
+                raise ValueError("Unknown date format! start_date: {}, end_date: {}".format(start_date, end_date))
         return dates_list
     elif func_name == "export":
         file_format, file_path = args[0], args[1]
@@ -335,5 +373,9 @@ def checks(self, func_name, *args):
         if not isinstance(is_visible, bool):
             raise ValueError('Param "is_visible" can only be boolean: True / False')
         return True
+    elif func_name == 'set_measure_direction':
+        is_horizontal = args[0]
+        if is_horizontal not in [True, False]:
+            raise ValueError('Param "is_horizontal" must be "True" or "False"!')
     else:
-        raise ValueError("No function to check: %s" % func_name)
+        raise ValueError("No function to check: {}".format(func_name))
