@@ -2,11 +2,11 @@
 # -*- coding: utf-8 -*-
 """ Содержит вспомогательные (служебные) методы, использующиеся в основном модуле работы с Полиматикой """
 
-from typing import List, Dict, Tuple
+from typing import Any, List, Dict, Tuple
 from itertools import count
 import time
 from .exceptions import ParseError, ScenarioError, PolymaticaException
-from .common import raise_exception
+from .common import raise_exception, FUNCS
 
 class Helper:
     def __init__(self, sc):
@@ -18,7 +18,7 @@ class Helper:
         # хранит функцию-генератор исключений
         self._raise_exception = raise_exception(self.sc)
 
-    def get_cube_id(self, cubes_list: List, cube_name: str) -> [str, bool]:
+    def get_cube_id(self, cubes_list: List, cube_name: str) -> str:
         """
         Получить id мультисферы (куба).
         :param cubes_list: список мультисфер
@@ -31,7 +31,7 @@ class Helper:
         return self._raise_exception(
             ValueError, 'No such cube "{}" in cubes list!'.format(cube_name), with_traceback=False)
 
-    def get_measure_or_dim_id(self, multisphere_data: Dict, measure_dim: str, name) -> [str, bool]:
+    def get_measure_or_dim_id(self, multisphere_data: Dict, measure_dim: str, name) -> str:
         """
         Получить id факта/размерности по имени.
         :param multisphere_data: рабочая область мультисферы
@@ -43,7 +43,8 @@ class Helper:
             if item["name"].rstrip() == name.rstrip():
                 return item["id"]
         error_msg = 'No such {}: "{}"'.format(measure_dim[:-1], name)
-        return self._raise_exception(ValueError, error_msg, with_traceback=False)
+        return self._raise_exception(
+            ValueError, 'No such {}: "{}"'.format(measure_dim[:-1], name), with_traceback=False)
 
     def get_dim_id(self, multisphere_data: Dict, name: str, cube_name: str) -> str:
         """
@@ -59,7 +60,7 @@ class Helper:
         error_msg = 'Dimension name "{}" is not valid for Multisphere "{}"!'.format(name, cube_name)
         return self._raise_exception(ValueError, error_msg, with_traceback=False)
 
-    def get_measure_id(self, multisphere_data: Dict, name: str, cube_name: str) -> [str, bool]:
+    def get_measure_id(self, multisphere_data: Dict, name: str, cube_name: str) -> str:
         """
         Получить идентификатор факта по его названию.
         :param multisphere_data: информация по рабочей области мультисферы.
@@ -95,13 +96,15 @@ class Helper:
         for item in data:
             if item.get("id") == id_:
                 return item.get("name")
-        return 'No {} with id "{}" in the multisphere!'.format('measure' if type_ == 'facts' else 'dimension', id_)
+        error_msg = 'No {} with id "{}" in the multisphere!'.format('measure' if type_ == 'facts' else 'dimension', id_)
+        return self._raise_exception(PolymaticaException, error_msg, with_traceback=False)
 
-    def get_measure_type(self, name: str) -> [bool, int]:
+    def get_measure_type(self, name: str) -> int:
         """
-        return Вид факта (value)
-        :param name: Вид факта
-        :return: Вид факта (id)
+        Возвращает целочисленный вид факта по его строковому названию. При передаче неверного строкового вида факта
+        будет сгенерирована ошибка.
+        :param name: (str) вид факта (в строковом представлении)
+        :return: (int) целочисленный вид факта.
         """
         measure_types = {
             "Значение": 0,
@@ -124,74 +127,47 @@ class Helper:
             return self._raise_exception(ValueError, "No such measure type: {}".format(name), with_traceback=False)
         return measure_types[name]
 
-    def join_splited_measures(self, formula_lst: List, join_iterations: int):
+    def parse_formula(self, formula: str) -> List:
         """
-        если в фактах есть пробелы, склеивает их обратно
-        :param formula_lst: (List) формула, разбитая на список (разделитель - пробел)
-        :param join_iterations: (int) количество итераций == количеству "[" в формуле
-        :return: (List) formula_lst со склееными фактами
+        Парсинг формулы для создания вычислимого факта.
+        Разбивает исходную формулу на составляющие, при этом склеивая названия фактов, содержащие пробелы.
+        Примеры:
+            1. "5 + [Сумма] * 2 + corr([Сумма];[ID])" -> ["5", "+", "[Сумма]", "*", "2", "+", "corr([Сумма];[ID])"]
+            2. "top([Сумма по всем отделам]; 10) >= 94" -> ["top([Сумма по всем отделам]; 10)", ">=", "94"]
+        :param formula: (str) исходная формула.
+        :return: (List) составляющие исходной формулы.
         """
-        # если кол-во итераций закончилось (факты закончились) - выйти из функции
-        if join_iterations == 0:
-            return formula_lst
-        # кол-во "[" и "]"
-        openning_sqr_braket = 0
-        closing_sqr_braket = 0
+        splitted_formula = formula.split()
+        result = []
+        in_func, in_measure, func_parts, measure_parts = False, False, list(), list()
 
-        # если все [факты с пробелами] и так склеены - вернуть formula_lst
-        for idx, elem in enumerate(formula_lst):
-            openning_sqr_braket += elem.count("[")
-            closing_sqr_braket += elem.count("]")
-            if openning_sqr_braket != closing_sqr_braket:
-                break
-            if (openning_sqr_braket == closing_sqr_braket) and (idx == len(formula_lst) - 1):
-                return formula_lst
-
-        # здесь будет сохранен [Факт, соединенный обратно по пробелу]
-        joined_measure = ""
-        # кол-во "[" и "]"
-        openning_sqr_braket = 0
-        closing_sqr_braket = 0
-        # индексы, которые занимает рассплитенный факт в списке:
-        measure_indexes = []
-
-        for idx, elem in enumerate(formula_lst):
-            # если факт и так не разделен по пробелу
-            # и если в формуле есть функции corr, top, total, if - переход к след. итерации
-            if ("[" in elem and "]" in elem) and ("corr(" not in joined_measure) and ("top(" not in joined_measure) and \
-                    ("total(" not in joined_measure) and ("if(" not in joined_measure):
+        for formula_part in splitted_formula:
+            # определяем, является ли текущая часть формулы функцией
+            if not in_func:
+                for logic_func in FUNCS:
+                    if "{}(".format(logic_func) in formula_part:
+                        in_func = True
+            if in_func:
+                func_parts.append(formula_part)
+                if ')' in formula_part:
+                    result.append(' '.join(func_parts))
+                    in_func, func_parts = False, list()
                 continue
-            openning_sqr_braket += elem.count("[")
-            closing_sqr_braket += elem.count("]")
-            # если в рассплитенном списке есть [Факт
-            # конкатенировать его с joined_measure
-            if openning_sqr_braket > 0:
-                joined_measure += elem
-                joined_measure += " "
-            # если факт в виде [Факт
-            # добавить индекс к списку measure_indexes
-            if (openning_sqr_braket != 0) and (closing_sqr_braket == 0):
-                measure_indexes.append(idx)
-            # если элемент == факт, и этот факт в виде [Факт]
-            # добавить индекс к списку measure_indexes и выйти из цикла
-            if (openning_sqr_braket != 0) and (closing_sqr_braket != 0):
-                if openning_sqr_braket == closing_sqr_braket:
-                    measure_indexes.append(idx)
-                    break
 
-        # удалить рассплитенный факт из formula_lst
-        for _ in measure_indexes:
-            formula_lst.pop(measure_indexes[0])
-        joined_measure = joined_measure.rstrip()
+            # определяем, является ли текущая часть формулы фактом
+            if not in_measure:
+                if '[' in formula_part:
+                    in_measure = True
+            if in_measure:
+                measure_parts.append(formula_part)
+                if ']' in formula_part:
+                    result.append(' '.join(measure_parts))
+                    in_measure, measure_parts = False, list()
+                continue
 
-        # добавить в formula_lst [Факт, соединенный обратно по пробелу]
-        formula_lst.insert(measure_indexes[0], joined_measure)
-
-        # уменьшить кол-во итераций (кол-во фактов) на 1
-        join_iterations -= 1
-
-        # рекурсивно вызвать join_splited_measures()
-        return self.join_splited_measures(formula_lst, join_iterations)
+            # всё остальное (операнды, числа) добавляем безо всяких преобразований
+            result.append(formula_part)
+        return result
 
     def get_scenario_id_by_name(self, script_descs: Dict, scenario_name: str) -> str:
         """
@@ -260,13 +236,13 @@ class Helper:
                 return self._raise_exception(ScenarioError, status_message, with_traceback=False)
         return count_of_requests, self.parse_result(result=progress, key="finished_steps_count")
 
-    def parse_result(self, result: Dict, key: str, nested_key: str = None):
+    def parse_result(self, result: Dict, key: str, nested_key: str = None) -> Any:
         """
         Парсит и проверяет на ошибки ответ в виде ['queries'][0]['command']['значение']['необязательное значение'].
-        :param result: (Dict) нераспарсенный ответ от API
-        :param key: (str) ключ, значение которого нужно распарсить
-        :param nested_key: (str) вложенный ключ, значение которого нужно распарсить
-        :return: итоговое значение
+        :param result: (Dict) нераспарсенный ответ от API.
+        :param key: (str) ключ, значение которого нужно распарсить.
+        :param nested_key: (str) вложенный ключ, значение которого нужно распарсить.
+        :return: (Dict) Значение заданного поля.
         """
         base_error_msg = "ERROR while parsing response: ['queries'][0]['command']"
         request_queries = result.get("queries")
@@ -291,26 +267,23 @@ class Helper:
         :param num_col: (int) количество колонок мультисферы
         :return: (Dict) command_name="view", state="get_2"
         """
-
         if (num_row is not None) and (num_col is not None):
-            return self.sc.execute_olap_command(command_name="view", state="get_2", from_row=0, from_col=0,
-                                                num_row=num_row,
-                                                num_col=num_col)
+            return self.sc.execute_olap_command(
+                command_name="view", state="get_2", from_row=0, from_col=0, num_row=num_row, num_col=num_col)
 
         # 1000, 2000, 3000, ...
         gen = count(1000, 1000)
 
         prev_data = []
 
-        result = self.sc.execute_olap_command(command_name="view", state="get_2", from_row=0, from_col=0,
-                                              num_row=next(gen),
-                                              num_col=next(gen))
+        result = self.sc.execute_olap_command(
+            command_name="view", state="get_2", from_row=0, from_col=0, num_row=next(gen), num_col=next(gen))
         data = self.parse_result(result=result, key="data")
 
         while len(prev_data) < len(data):
             prev_data = data
-            result = self.sc.execute_olap_command(command_name="view", state="get_2", from_row=0, from_col=0,
-                                                  num_row=next(gen), num_col=next(gen))
+            result = self.sc.execute_olap_command(
+                command_name="view", state="get_2", from_row=0, from_col=0, num_row=next(gen), num_col=next(gen))
             data = self.parse_result(result=result, key="data")
         return result
 
